import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.List;
import java.util.zip.GZIPOutputStream;
import java.io.File;
import org.apache.axis2.AxisFault;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import com.ebay.marketplace.services.*;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

public class LMSSample {
	private static DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss.SSS");
	
	private static String getJobTypeFromXML(String inFilename) {
		// get the JobType from the input xml file
		File inputXml = null;
		inputXml = new File(inFilename);
		LMSRequestHandler bdxHandler = new LMSRequestHandler();
		CreateLMSParser parser = new CreateLMSParser(bdxHandler);
		boolean parseOk = parser.parse(inputXml);
		if (!parseOk) {
			System.out.println("Failed to extract the JobType from the file ["
					+ inFilename + "]");
			return null;
		}

		// extract the JObType String successfully
		String jobType = bdxHandler.getJobType();
		if (jobType == null)
			System.out.println("Invalid job type in the XML file.");
		else System.out.println("Found the job type from the XML file, it is "
				+ jobType);
		return jobType;
	}

	private static String compressFileToGzip(String inFilename) {
		// compress the xml file into gz file in the save folder
		String outFilename = null;
		String usingPath = inFilename.substring(0, inFilename
				.lastIndexOf(File.separator) + 1);
		String fileName = inFilename.substring(inFilename
				.lastIndexOf(File.separator) + 1);
		outFilename = usingPath + fileName + ".gz";

		try {
			BufferedReader in = new BufferedReader(new FileReader(inFilename));
			BufferedOutputStream out = new BufferedOutputStream(
					new GZIPOutputStream(new FileOutputStream(outFilename)));
			System.out.println("Writing gz file...");
			int c;
			while ((c = in.read()) != -1)
				out.write(c);
			in.close();
			out.close();
		} catch (FileNotFoundException e) {
			System.out.println("Cannot find file: " + inFilename);
		} catch (IOException e) {
			System.out.println("IOException:" + e.toString());
		}
		System.out.println("The compressed file has been saved to " + outFilename);
		return outFilename;
	}
	
	/**
	 * The method demonstrate how to upload file attachment using XML over HTTP (MIME attachment)
	 */
	private static void downloadFileUseHTTP(String fileSaveTo, String task_ref_id , String file_ref_id){
		HttpClient client = new HttpClient();
		PostMethod method = new PostMethod(LMSProperties.getInstance().getFtsEndPointURL());
		
		String xmlStart = "<?xml version='1.0' encoding='UTF-8'?><downloadFileRequest xmlns:sct=\"http://www.ebay.com/soaframework/common/types\" xmlns=\"http://www.ebay.com/marketplace/services\">";
		String taskRefId =  "<taskReferenceId>" + task_ref_id + "</taskReferenceId>";
		String fileRefId = "<fileReferenceId>" + file_ref_id + "</fileReferenceId>";
		String xmlEnd = "</downloadFileRequest>";

		String payLoad = xmlStart + taskRefId + fileRefId + xmlEnd;
		try {
			String tokenString = LMSProperties.getInstance().getAuthToken();
			
			byte[] requestBytes = payLoad.getBytes("UTF-8");
			ByteArrayRequestEntity be = new ByteArrayRequestEntity(requestBytes);
			method.setRequestEntity(be);
			
			String contentType = "application/soap+xml;charset=UTF-8;action=\"urn:downloadFile\"";
			method.setRequestHeader("Content-type", contentType);
			method.setRequestHeader("X-EBAY-SOA-SERVICE-NAME", "FileTransferService");
			method.setRequestHeader("X-EBAY-SOA-OPERATION-NAME", "downloadFile");
			method.setRequestHeader("X-EBAY-SOA-SECURITY-TOKEN", tokenString);
			method.setRequestHeader("X-EBAY-SOA-REQUEST-DATA-FORMAT", "XML");
			method.setRequestHeader("X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		} catch (Exception e) {
			System.err.println("Exception is: " + e.getLocalizedMessage());
		}

		try {

			int statusCode = client.executeMethod(method);

			if (statusCode != HttpStatus.SC_OK) {
				System.err.println("Method failed: " + method.getStatusLine());
			}

			String responseContent = null;
			try {
				responseContent = method.getResponseBodyAsString();
			} catch (NullPointerException e) {
				System.out.println("NullPointerException caught.");
			}

			String ack = null;
			int index_ack_s = responseContent.indexOf("<ack>");
			int index_ack_e = responseContent.indexOf("</ack>");
			ack = responseContent.substring(index_ack_s+"<ack>".length(), index_ack_e);
			System.out.println("ack="+ack);
			if (ack.trim().toLowerCase().equals("success")){
				//if ack = Success, then get the attachment out and save it to file
				int index_uuid_s = responseContent.indexOf("<Data><xop:Include xmlns:xop=\"http://www.w3.org/2004/08/xop/include\" href=\"cid:urn:uuid:");
				int index_uuid_e = responseContent.indexOf("\" /></Data></fileAttachment>");
				String cid_uuid= responseContent.substring(index_uuid_s+"<Data><xop:Include xmlns:xop=\"http://www.w3.org/2004/08/xop/include\" href=\"cid:urn:uuid:".length(), index_uuid_e);
//				System.out.println("cid_uuid="+cid_uuid);
				String start_flag="Content-ID: <urn:uuid:"+cid_uuid+">";
				int index_start=responseContent.indexOf(start_flag)+start_flag.length()+4;
				int index_end = responseContent.lastIndexOf("--MIMEBoundaryurn_uuid_");
				
				byte[] bytesForFile	= new byte[index_end-index_start];
				byte[] responseBytes = method.getResponseBody();
				for (int i=0;i<index_end-index_start;i++){
					bytesForFile[i]=responseBytes[i+index_start];
				}
				if (bytesForFile==null) System.err.println("bytesForFile is null");
				FileOutputStream fos = new FileOutputStream(fileSaveTo);
				fos.write(bytesForFile);
				fos.close(); 
			}else{
				//if ack = failed then print the error message out
				int index_xml_s = responseContent.indexOf("<downloadFileResponse");
				int index_xml_e = responseContent.indexOf("</downloadFileResponse>");
				String xmlString = responseContent.substring(index_xml_s, index_xml_e+"</downloadFileResponse>".length());
				System.out.println(xmlString);
			}
		} catch (HttpException e) {
			System.err.println("Fatal protocol violation: " + e.getMessage());
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Fatal transport error or File error: " + e.getMessage());
			e.printStackTrace();
		} finally {
			// Release the connection.
			method.releaseConnection();
		}
	}
	
	
	/**
	 * The method demonstrate how to upload file attachment using XML over HTTP (MIME attachment)
	 */
	private static void uploadFileUseHTTP(String attachmentFile, String task_ref_id , String file_ref_id ){
		HttpClient client = new HttpClient();
		PostMethod method = new PostMethod(LMSProperties.getInstance().getFtsEndPointURL());
		
		String xmlStart = "<?xml version='1.0' encoding='UTF-8'?><uploadFileRequest xmlns:sct=\"http://www.ebay.com/soaframework/common/types\" xmlns=\"http://www.ebay.com/marketplace/services\">";
		String taskRefId =  "<taskReferenceId>" + task_ref_id + "</taskReferenceId>";
		String fileRefId = "<fileReferenceId>" + file_ref_id + "</fileReferenceId>";
		String xmlEnd = "<fileFormat>gzip</fileFormat><fileAttachment><Data><xop:Include xmlns:xop=\"http://www.w3.org/2004/08/xop/include\" href=\"cid:urn:uuid:EE69C2F1D5C814ADF71222389408640\" /></Data> </fileAttachment></uploadFileRequest>";

		String payLoad = xmlStart + taskRefId + fileRefId + xmlEnd;
		
		System.out.println("The Payload is::: \n" + payLoad);
		
		try {
			String tokenString = LMSProperties.getInstance().getAuthToken();
			
			byte[] multipartRequest = MIMEUtils.createMultiPartRequest(
					payLoad, attachmentFile);
			ByteArrayRequestEntity be = new ByteArrayRequestEntity(
					multipartRequest);
			method.setRequestEntity(be);//asssociate the file attachement to the Post method

			String contentType = "multipart/related; boundary="
					+ MIMEUtils.getBoundary()
					+ ";type=\"application/xop+xml\";"
					+ "start=\"<0.urn:uuid:883CA9C91CE9C469D912223894056542>\";"
					+ "start-info=\"text/xml\"";

			method.setRequestHeader("Content-type", contentType);

			method.setRequestHeader("X-EBAY-SOA-SERVICE-NAME", "FileTransferService");
			method.setRequestHeader("X-EBAY-SOA-OPERATION-NAME", "uploadFile");
			method.setRequestHeader("X-EBAY-SOA-SECURITY-TOKEN", tokenString);
			method.setRequestHeader("X-EBAY-SOA-REQUEST-DATA-FORMAT", "XML");
			method.setRequestHeader("X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		} catch (Exception e) {
			System.err.println("Exception is: " + e.getLocalizedMessage());
		}

		try {

			int statusCode = client.executeMethod(method);

			if (statusCode != HttpStatus.SC_OK) {
				System.err.println("Method failed: " + method.getStatusLine());
			}

			String responseContent = null;
			try {
				responseContent = method.getResponseBodyAsString();
			} catch (NullPointerException e) {
				// do graceful stuff
			}

			int responseCode = method.getStatusCode();
			String responseMess = method.getStatusText();

			System.out.println("Response Content is::: " + responseContent);
			System.out.println("Response Code is::: " + responseCode);
			System.out.println("Response Message is::: " + responseMess);

			// Read the response body.
			// byte[] responseBody = method.getResponseBody();

			// Deal with the response.
			// Use caution: ensure correct character encoding and is not binary data

		} catch (HttpException e) {
			System.err.println("Fatal protocol violation: " + e.getMessage());
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Fatal transport error: " + e.getMessage());
			e.printStackTrace();
		} finally {
			// Release the connection.
			method.releaseConnection();
		}
	}
	
	/* 
	 * main entry point of the program
	 * 
	 */
	public static void main(String args[]) throws AxisFault, RemoteException,
			Exception {
		if (args.length < 1) {
			// if no parameters are given, then it print out the description of available commands 
			StringBuilder sb = new StringBuilder("");
			sb.append("LMSSample sample commands\n");
			sb.append("=========================\n");
			sb.append("#1: java -jar LMSSample.jar UploadJobEnd2End D:\\ReviseFixedPriceItem.xml D:\\download1.zip\n");
			sb.append("-------------------\n");
			sb.append("Input parameters\n");
			sb.append("1.Action String (UploadJobEnd2End)\n");
			sb.append("2.Location of XML file to upload (D:\\ReviseFixedPriceItem.xml)\n");
			sb.append("3.File name for the download attachment (D:\\download1.zip)\n");
			sb.append("\n");
			sb.append("This command does the following steps:\n");
			sb.append("1.Get the JobType from the input XML file\n");
			sb.append("2.Compress the input XML file into gzip file\n");
			sb.append("3.Call BDX createUploadloadJob service to create a job\n");
			sb.append("4.Call FTS uploadFile service to upload the gzip file that was created in Step #2\n");
			sb.append("5.Call BDX startUploadloadJob service to start the job that was created in Step #3\n");
			sb.append("6.Call BDX getJobStatus service to see if the Job is completed\n");
			sb.append("7.If the JobStatus returned is not yet Completed,  then sleep for 20 seconds (configurable in BDX.properties) and repeat Step #5. When the JobStatus returns Completed, move to Step #7 \n");
			sb.append("8.Call FTS downloadFile service to download the result file that is specified by the 3rd input parameter\n");
			sb.append("\n");
			sb.append("#2: java -jar LMSSample.jar UploadFile 5000000636 50000000236 D:\\ReviseFixedPriceItem.xml useSOAP/useHTTP \n");
			sb.append("-------------------\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (UploadFile)\n");
			sb.append("2.TaskRefId (This is returned in the createUploadloadJob service call)\n");
			sb.append("3.FileRefId (This is returned in the createUploadloadJob service call)\n");
			sb.append("4.Full file name of the XML file\n");
			sb.append("5.useSOAP or useHTTP to indicate which protocol the invocation will use\n");
			sb.append("This command does the following steps:\n");
			sb.append("1.Compress the input XML file into gzip file\n");
			sb.append("2.Call FTS uploadFile service to upload file where JobID is the 2nd parameter and FileReferenceID is the 3rd parameter \n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#3: java -jar LMSSample.jar DownloadFile 5000000636 50000000236 D:\\downloadResult.zip useSOAP/useHTTP \n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (DownloadFile)\n");
			sb.append("2.TaskRefId (This is returned in the createUploadloadJob service call)\n");
			sb.append("3.FileRefId (This is returned in the createUploadloadJob service call)\n");
			sb.append("4.Full file name of the file it will save locally\n");
			sb.append("5.useSOAP or useHTTP to indicate which protocol the invocation will use\n");
			sb.append("This command does the following steps:\n");
			sb.append("1.Call FTS downloadFile to download file for the job where JobID is the 2nd parameter and FileReferenceID is the 3rd parameter \n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#4: java -jar LMSSample.jar getJobs creationTimeFrom=2008-09-01&creationTimeTo=2008-10-02&jobType=RelistFixedPriceItem&jobStatus=Failed\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (getJobs)\n");
			sb.append("2.query criteria string\n");
			sb.append("This command calls BDX getJobs service to get job profiles that satisfy the query criteria.\n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#5: java -jar LMSSample.jar abortJob 5000000636\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (abortJob)\n");
			sb.append("2.JobID\n");
			sb.append("This command calls BDX abortJob service to abort an unterminated job where JobID is the 2nd parameter.\n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#6: java -jar LMSSample.jar createUploadJob ReviseFixedPriceItem\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (createUploadJob)\n");
			sb.append("2.JobType\n");
			sb.append("This command calls BDX createUploadJob service to create an upload job with the given JobType (e.g. ReviseFixedPriceItem).\n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#7: java -jar LMSSample.jar startUploadJob 5000000636\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (startUploadJob)\n");
			sb.append("2.JobID\n");
			sb.append("This command calls BDX startUploadJob service to start the created upload job where JobID is the 2nd parameter.\n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#8: java -jar LMSSample.jar getJobStatus 5000000636\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (getJobStatus)\n");
			sb.append("2.JobID\n");
			sb.append("This command calls BDX getJobStatus service to query the job status where JobID is the 2nd parameter.\n");
			sb.append("\n");
			sb.append("\n");
			sb.append("#9: a> java -jar LMSSample.jar startDownloadJob SoldReport\n");
			sb.append("    b> java -jar LMSSample.jar startDownloadJob SoldReport 2008-10-10_12:0:0\n");
			sb.append("\n");
			sb.append("Input parameters:\n");
			sb.append("1.Action String (startDownloadJob)\n");
			sb.append("2.JobType (ActiveInventoryReport, SoldReport, or FeeSettlementReport)\n");
			sb.append("3.startTime (Optional) format = \"yyyy-mm-dd_hh:mm:ss\"\n");
			sb.append("This command calls BDX startDownloadJob service to start the download job, it returns a JobID.\n");
			System.out.println(sb.toString());
			return;
		}

		String action = args[0];
		// End to end uploadJob
		if (action.toLowerCase().equals("UploadJobEnd2End".toLowerCase())) {
			String uploadFileName, downloadFileName;
			uploadFileName = args[1];
			downloadFileName = args[2];
			if (end2EndUploadJob(uploadFileName, downloadFileName))
				System.out.println("\n******\nUploadJobEnd2End finished successfully.");
			else
				System.out.println("UploadJobEnd2End failed.");

		// FTS uploadFile
		} else if (action.toLowerCase().equals("UploadFile".toLowerCase())) {
			if (args.length < 5) {
				System.out
						.println("Sample command: java -jar LMSSample.jar UploadFile 5000000636 50000000236 D:\\AddFixedPriceItem.xml useSOAP");
				return;
			}
			
			String taskRefId, fileRefId, uploadXML, fileToUpload, protocol;
			taskRefId = args[1];
			fileRefId = args[2];
			uploadXML = args[3];
			protocol  = args[4];
			
			fileToUpload = compressFileToGzip(uploadXML);
			if (fileToUpload == null) {
				System.out.println("Failed to compress file:" + uploadXML
						+ " to gzip file, abort.");
				return;
			}
			
			if ("usesoap".equals(protocol.trim().toLowerCase())){
				System.out.println("Now use SOAP to invoke the uploadFile operation.");
				FTSServiceClient.uploadFile("gzip", taskRefId, fileRefId, fileToUpload);
			}else if ("usehttp".equals(protocol.trim().toLowerCase())){
				System.out.println("Now use XML+HTTP to invoke the uploadFile operation.");
				uploadFileUseHTTP(fileToUpload,taskRefId, fileRefId);
			}else {
				System.out.println("Please enter the valid protocol option, useHTTP or useSOAP.");
			}
		} 
		// FTS downloadFile
		else if (action.toLowerCase().equals("DownloadFile".toLowerCase())) {
			if (args.length < 5) {
				System.out
						.println("Sample command: java -jar LMSSample.jar DownloadFile 5000000636 50000000236 D:\\downloadResult.gz useHTTP");
				return;
			}
			String taskRefId, fileRefId, downloadFileTo,protocol;
			taskRefId = args[1];
			fileRefId = args[2];
			downloadFileTo = args[3];
			protocol  = args[4];
			
			if ("usesoap".equals(protocol.trim().toLowerCase())){
				System.out.println("Now use SOAP to invoke the downloadFile operation.");
				FTSServiceClient.downloadFile(downloadFileTo, taskRefId, fileRefId);
			}else if ("usehttp".equals(protocol.trim().toLowerCase())){
				System.out.println("Now use HTTP+XML to invoke the downloadFile operation.");
				downloadFileUseHTTP(downloadFileTo, taskRefId, fileRefId);
			}else {
				System.out.println("Please enter the valid protocol option, useHTTP or useSOAP.");
			}
		}  
//		BDX getJobs
		else if (action.trim().toLowerCase().equals("getjobs")) {
			if (args.length < 2) {
				System.out
						.println("Sample command: java -jar LMSSample.jar getJobs creationTimeFrom=2008-09-01&creationTimeTo=2008-10-02&jobType=RelistFixedPriceItem&jobStatus=Failed");
				return;
			}
			String conditionsStr;
			conditionsStr = args[1];
			LMSServiceClient.getJobs(conditionsStr);
		} 
			//BDX abortJob
		else if (action.trim().toLowerCase().equals("abortjob")) {
			if (args.length < 2) {
				System.out
						.println("Sample command: java -jar LMSSample.jar abortJob 5000001895");
				return;
			}
			String jobID = args[1];
			LMSServiceClient.abortJob(jobID);
		} 
			//BDX startUploadJob
		else if (action.trim().toLowerCase().equals("startUploadJob".toLowerCase())){
			if (args.length < 2) {
				System.out
						.println("Sample command: java -jar LMSSample.jar startUploadJob 5000001895");
				return;
			}
			String jobID = args[1];
			LMSServiceClient.startUploadJob(jobID);
		}
			//BDX createUploadJob
		else if (action.trim().toLowerCase().equals("createUploadJob".toLowerCase())){
			if (args.length < 2) {
				System.out
						.println("Sample command: java -jar LMSSample.jar createUploadJob AddFixedPriceItem");
				return;
			}
			String jobType = args[1];
			LMSServiceClient.createUploadJob(jobType);
		}
			//BDX getJobStatus
		else if (action.trim().toLowerCase().equals("getJobStatus".toLowerCase())){
			if (args.length < 2) {
				System.out
						.println("Sample command: java -jar LMSSample.jar getJobStatus 5000001895");
				return;
			}
			String inputJobId = args[1];
			LMSServiceClient.getJobStatus(inputJobId, false); // for the 2nd argument, true or false doesn't matter, the print out of the service response  is done within the the method itself.
		} // startDownloadJob
		else if (action.trim().toLowerCase().equals(
				"startDownloadJob".toLowerCase())) {
			if (args.length < 2){
				System.out
						.println("Sample command #1: java -jar LMSSample.jar startDownloadJob SoldReport");
			System.out
					.println("Sample command #2: java -jar LMSSample.jar startDownloadJob SoldReport 2008-10-10_12:0:0");
			return;
			}
			if (args.length == 2) 
				LMSServiceClient.startDownloadJob(args[1], null);
			if (args.length == 3) 
				LMSServiceClient.startDownloadJob(args[1], args[2]);
		}
		
		else {
			//	handle invalid input task name
			System.out
					.println("The first argument has to be one of [UploadJobEnd2End, UploadFile or DownloadFile, getJobs, abortJob, createUploadJob, startUploadJob, getJobStatus, startDownloadJob]");
			return;
		}
	}

	// implementation of the end2EndUploadJob command
	static private boolean end2EndUploadJob(String uploadFileName,
			String downloadFileName) throws Exception {
		String newJobId=null;
		String newFileRefId=null;
		// verify file to upload
		BufferedReader file;
		try {
			file = new BufferedReader(new FileReader(uploadFileName));
			String st = file.readLine();
			System.out.println("The first line of "+uploadFileName+" is:"+st);
			System.out.println("File existence check passed.");
		} catch (FileNotFoundException e) {
			System.out.println("File '" + uploadFileName + "' not found.");
			return false;
		} catch (IOException e) {
			return false;
		}

		// get JobType from the XML file
		String jobType = getJobTypeFromXML(uploadFileName);

		if (jobType == null)
			return false;

		// compress the input XML file into gzip file

		String uploadGzipFileName = compressFileToGzip(uploadFileName);
		if (uploadGzipFileName == null) {
			System.out
					.println("\nFailed to compress your XML file into gzip file. Aborted.");
			return false;
		}

		// create upload job
		CreateUploadJobResponse cujr = LMSServiceClient
				.createUploadJob(jobType);
		if (cujr == null) {
			System.out.println("\nCreateUploadJobResponse was NULL");
			return false;
		}
		newJobId = cujr.getJobId();
		newFileRefId = cujr.getFileReferenceId();
		if (newFileRefId == null || newJobId == null || newFileRefId.length() == 0
				|| newJobId.length() == 0) {
			System.out
					.println("The createUploadJob request didn't return job ID and file reference ID. UploadJobEnd2End command halted.");
			return false;
		}
		// start upload file
		System.out.println("\nnow upload file .......");
		boolean uploadOK = FTSServiceClient.uploadFile("gzip", newJobId,
				newFileRefId, uploadGzipFileName);

		if (!uploadOK) {
			System.out.println("The file uploading failed, UploadJobEnd2End command halted.");
			return false;
		}

		// start uploaded job
		System.out.println("\nInvoking startUploadJob");
		StartUploadJobResponse sujr = LMSServiceClient
				.startUploadJob(newJobId);
		if (sujr == null) {
			System.out.println("StartUploadJobResponse was NULL, the startUploadJob failed, UploadJobEnd2End command halted.");
			return false;
		} else {
			// the client will print out the response ACK, so no need to print
			// again here;
			if (sujr.getAck().equals("Failure")){
				System.out.println("The startUploadJob failed, UploadJobEnd2End command halted.");
				return false;
			}
		}

		// loop querying the JobStatus

		System.out.println("\nLooping getJobStatus (wait for the job to be processed)\n");
		System.out.println(dateFormat.format(new java.util.Date()));
		JobStatus jobStatus = (JobStatus) LMSServiceClient
				.getJobStatus(newJobId, false);
		
		while (!jobStatus.equals(JobStatus.COMPLETED) && !jobStatus.equals(JobStatus.FAILED) && !jobStatus.equals(JobStatus.ABORTED)){
			Thread.sleep(LMSProperties.getInstance().getGetJobStatusQueryInterval());
			System.out.println();
			System.out.println(dateFormat.format(new java.util.Date()));
			jobStatus = (JobStatus) LMSServiceClient.getJobStatus(newJobId, false);
		}
		
		System.out.println();
		// job is terminated, if the status is Completed, then get the output file_reference_id for downloading the file
		if (jobStatus.equals(JobStatus.COMPLETED)){
			GetJobStatusResponse rsp = (GetJobStatusResponse) LMSServiceClient.getJobStatus(newJobId, true);
			AckValue ackValue = rsp.getAck();
			String fileRefIdForDownload = null;
			if ("Success".equals(ackValue.value())) {
				System.out.println("\nThe job completed, now download the result file...");
				List<JobProfile> profiles = rsp.getJobProfile();
				JobProfile profile = profiles.get(0);
				fileRefIdForDownload = profile.getFileReferenceId();
			}
			if (fileRefIdForDownload!=null)
				return FTSServiceClient.downloadFile(downloadFileName, newJobId, fileRefIdForDownload);
			else
				return false;
		}else if (jobStatus.equals(JobStatus.COMPLETED)){
			System.out.println("The job has been aborted.");
		}else if (jobStatus.equals("Failed")){
			System.out.println("The job failed.");
		}
		return true;

	} // end of method
	
}
