

import java.rmi.RemoteException;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.axis2.context.MessageContext;
import com.ebay.marketplace.services.*;
import com.ebay.www.marketplace.services.BulkDataExchangeServiceStub;

import java.util.UUID;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.text.ParseException;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

/*
 * BDXServiceClient wrap up the service stub class to make invocation easier
 */

public class LMSServiceClient extends BulkDataExchangeServiceStub {
	private LMSServiceClient(String endPointURL)
			throws org.apache.axis2.AxisFault {
		super(endPointURL);
	}

	/*
	 * getPairs method parse the getJobs condition String, and put the name
	 * value pair in to Map object for easier accessing. Sample condition string
	 * can be:
	 * "creationTimeFrom=2008-09-01&creationTimeTo=2008-10-02&jobType=RelistItem&jobStatus=Failed"
	 * if the delimited char is "&"
	 */
	public static Map<String, String> getPairs(String pairs, String splitStr) {
		if (pairs == null || pairs.equals("")) {
			return null;
		}
		String[] reqFields = pairs.split(splitStr);
		Map<String, String> parameterMap = new HashMap<String, String>();
		for (int i = 0; i < reqFields.length; i++) {
			String nameValuepair = reqFields[i];
			String[] nameValueArray = nameValuepair.split("=");
			parameterMap.put(nameValueArray[0].trim(),
					nameValueArray.length <= 1 ? null : nameValueArray[1]
							.trim());
		}
		return parameterMap;
	}

	/*
	 * This method is used to pretty print the ErrorMessage object returned from
	 * the service response.
	 */
	private static String printErrors(ErrorMessage errorMessage) {
		System.out.println("Error messages return from the service call are:");
		StringBuilder builder = new StringBuilder("");
		if (errorMessage != null) {
			List<ErrorData> errors = errorMessage.getError();
			for (ErrorData ed : errors) {
				builder.append(ed.getMessage());
				builder.append("\n");
			}
		}
		return builder.toString();
	}

	/*
	 * This method represent the getJobStatus service operation.
	 */
	public static Object getJobStatus(String inputJobId,
			boolean returnResponseObject) {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		JobStatus jobStatus = null;
		LMSServiceClient testStub = null;
		GetJobStatusResponse getJobStatusRsp = null;
		try {
			MessageContext msgCtx = new MessageContext();
			testStub = new LMSServiceClient(endPointURL);
			testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
					"getJobStatus");
			testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN",
					authToken);
			testStub
					.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
							"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
			testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL",
					"SOAP11");
			testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT",
					"SOAP");
			testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT",
					"XML");

			GetJobStatusRequest getJobStatusReq = new GetJobStatusRequest();
			System.out.println("Call getJobStatus for jobID=" + inputJobId);
			getJobStatusReq.setJobId(inputJobId);
			getJobStatusRsp = testStub.getJobStatus(
					getJobStatusReq, msgCtx);
			AckValue ackValue = getJobStatusRsp.getAck();
			System.out.println("Ack=" + ackValue.value());
			if ("Success".equals(ackValue.value())) {
				List<JobProfile> profiles = getJobStatusRsp.getJobProfile();
				System.out.println("The returned JobProfile is:");
				JobProfile profile = profiles.get(0);
				printJobProFile(profile);
				jobStatus = profile.getJobStatus();
			} else {
				ErrorMessage msg = getJobStatusRsp.getErrorMessage();
				System.out.println("\n" + printErrors(msg));
			}
		} catch (RemoteException e) {
			System.out.println("Job is in process.");
			return "error";
		}
		testStub = null;
		if (returnResponseObject)
			return getJobStatusRsp;
		else
			return jobStatus;
	}

	/*
	 * This method represent the createUploadJob service operation.
	 */
	public static CreateUploadJobResponse createUploadJob(String uploadJobtype)
			throws RemoteException {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		LMSServiceClient testStub = new LMSServiceClient(endPointURL);
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
				"createUploadJob");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP11");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		CreateUploadJobRequest req = new CreateUploadJobRequest();
		System.out.println("Call createUploadJob...");
		req.setUploadJobType(uploadJobtype);
		req.setUUID(UUID.randomUUID().toString());
		req.setFileType(FileType.XML);
		CreateUploadJobResponse rsp = testStub.createUploadJob(req, msgCtx);

		AckValue ackValue = rsp.getAck();
		System.out.println("Ack=" + ackValue.value());

		if ("Success".equals(ackValue.value())) {
			System.out.println("JobId=" + rsp.getJobId() + "\nFileReferenceId="
					+ rsp.getFileReferenceId());
		} else {
			ErrorMessage msg = rsp.getErrorMessage();
			System.out.println("\n" + printErrors(msg));
		}
		return rsp;
	}

	/*
	 * This method represent the startUploadJob service operation.
	 */
	public static StartUploadJobResponse startUploadJob(String jobId)
			throws RemoteException {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		LMSServiceClient testStub = new LMSServiceClient(endPointURL);
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
				"startUploadJob");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP11");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		StartUploadJobRequest req = new StartUploadJobRequest();
		System.out.println("Call startUploadJob...");
		req.setJobId(jobId);
		StartUploadJobResponse rsp = testStub.startUploadJob(req, msgCtx);

		AckValue ackValue = rsp.getAck();
		System.out.println("Ack=" + ackValue.value());
		if ("Failure".equals(ackValue.value())) {
			ErrorMessage msg = rsp.getErrorMessage();
			System.out.println("\n" + printErrors(msg));
		}
		testStub = null;
		return rsp;
	}

	
	/**
	 * This method represent the startDownloadJob service operation.
	 */
	public static StartDownloadJobResponse startDownloadJob(String jobType, String inputDateString)
			throws RemoteException {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		LMSServiceClient testStub = new LMSServiceClient(endPointURL);
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME",
				"startDownloadJob");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP11");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		StartDownloadJobRequest req = new StartDownloadJobRequest();
		System.out.println("Call startDownloadJob...");
		req.setDownloadJobType(jobType);
		if (inputDateString != null) {
			FeeSettlementReportFilter feeSettlementReportFilter = new FeeSettlementReportFilter();
			XMLGregorianCalendar startTime;
			SimpleDateFormat df = new SimpleDateFormat("yyyy-mm-dd_hh:mm:ss");
			
			try {
				startTime = parseDateTime(inputDateString);
			} catch (DatatypeConfigurationException e) {
				System.out
						.println("Could not convert the input time to Calendar, please check your date format, it should be yyyy-mm-dd_hh:mm:ss");
				return null;
			}
			if (startTime==null){
				System.out.println("Could not convert the input time to Calendar, please check your date format, it should be yyyy-mm-dd_hh:mm:ss");
				return null;
			}
			feeSettlementReportFilter.setStartTime(startTime);
			DownloadRequestFilter downloadRequestFilter = new DownloadRequestFilter();
			downloadRequestFilter
					.setFeeSettlementReportFilter(feeSettlementReportFilter);
			req.setDownloadRequestFilter(downloadRequestFilter);
		}
		req.setUUID( java.util.UUID.randomUUID().toString());
		StartDownloadJobResponse rsp = testStub.startDownloadJob(req, msgCtx);

		AckValue ackValue = rsp.getAck();
		System.out.println("Ack=" + ackValue.value());
		
		if ("Failure".equals(ackValue.value())) {
			ErrorMessage msg = rsp.getErrorMessage();
			System.out.println("\n" + printErrors(msg));
		}else {
			System.out.println("JobId="+rsp.getJobId());
		}
		testStub = null;
		return rsp;
	}
	
		
	/*
	 * This method represent the getJobs service operation.
	 */
	public static GetJobsResponse getJobs(String conditionsStr)
			throws RemoteException, DatatypeConfigurationException {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		LMSServiceClient testStub = new LMSServiceClient(endPointURL);
		// setting the HTTP header
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME", "getJobs");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP11");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		GetJobsRequest req = new GetJobsRequest();
		System.out.println("Call getJobs...");

		XMLGregorianCalendar fromCal = null;
		// validating the input conditions
		XMLGregorianCalendar toCal = null;
		java.util.Map<String, String> conditions = getPairs(conditionsStr, "&");
		if (conditions.get("creationTimeFrom") != null
				&& !conditions.get("creationTimeFrom").equals("")) {
			fromCal = parseDate(conditions.get("creationTimeFrom"));
			if (fromCal == null){
				System.out.println("Criteria creationTimeFrom has been ignored.");
			}
			req.setCreationTimeFrom(fromCal);
		}
		if (conditions.get("creationTimeTo") != null
				&& !conditions.get("creationTimeTo").equals("")) {
			toCal = parseDate(conditions.get("creationTimeTo"));
			if (toCal == null){
				System.out.println("Criteria creationTimeTo has been ignored.");
			}
			req.setCreationTimeTo(toCal);
		}
		
		if (conditions.get("jobType") != null
				&& !conditions.get("jobType").equals("")) {
			List<String> jobType = req.getJobType();
			jobType.add(conditions.get("jobType"));
		}

		if (conditions.get("jobStatus") != null
				&& !conditions.get("jobStatus").equals("")) {
			List<JobStatus> jobStatuss = req.getJobStatus();
			try {
				jobStatuss.add(JobStatus.fromValue(conditions.get("jobStatus")))  ;
			} catch (java.lang.IllegalArgumentException e) {
				System.out
						.println("Service call failed on the client side. \nThe input jobStatus is not valid, please check it and try again.");
				return null;
			}
		}

		// invoke
		GetJobsResponse rsp = testStub.getJobs(req, msgCtx);

		// handling the response
		AckValue ackValue = rsp.getAck();
		System.out.println("Ack=" + ackValue.value());
		if ("Success".equals(ackValue.value())) {
			List<JobProfile> profiles = rsp.getJobProfile();
			if (profiles != null) {
				System.out.println("The returned JobProfiles are:");
				int i = 1;
				for (JobProfile profile : profiles) {
					System.out.println();
					System.out.println("JobProfile #" + i++);
					printJobProFile(profile);
				}
			} else
				System.out
						.println("No jobs match your query criteria.");
		} else {
			ErrorMessage msg = rsp.getErrorMessage();
			System.out.println("\n" + printErrors(msg));
		}
		testStub = null;
		return rsp;
	}

	/**
	 * This mathod print the JobProfile prettyly
	 * @param profile
	 * @return
	 */
	
	private static void printJobProFile(JobProfile profile){
		
		System.out.println("jobId:\t\t" + profile.getJobId());
		System.out.println("jobType:\t" + profile.getJobType());
		System.out.println("jobStatus:\t" + profile.getJobStatus());
		System.out.println("creationTime:\t"
				+ (profile.getCreationTime() == null ? "Null"
						: (profile.getCreationTime().toXMLFormat()))); 
		if (profile.getCompletionTime() != null)
			System.out.println("completionTime:\t" + (profile.getCompletionTime().toXMLFormat())); 
		if (profile.getErrorCount() != null)
			System.out.println("errorCount:\t"+ profile.getErrorCount().intValue());
		if (profile.getPercentComplete() != null) 
			System.out.println("PercentComplete:" + profile.getPercentComplete().doubleValue());
		if (profile.getFileReferenceId() != null)
			System.out.println("fileReferenceId:"+ profile.getFileReferenceId());;
	}
	
	/*
	 * This method represent the abortJob service operation.
	 */
	public static AbortJobResponse abortJob(String jobID)
			throws RemoteException {
		String authToken = LMSProperties.getInstance().getAuthToken();
		String endPointURL = LMSProperties.getInstance().getBdxEndPointURL();
		LMSServiceClient testStub = new LMSServiceClient(endPointURL);
		// setting HTTP headers
		MessageContext msgCtx = new MessageContext();
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-OPERATION-NAME", "abortJob");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-SECURITY-TOKEN", authToken);
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-SERVICE-NAME",
						"{http://www.ebay.com/marketplace/services}BulkDataExchangeService");
		testStub.addHttpHeader(msgCtx, "X-EBAY-SOA-MESSAGE-PROTOCOL", "SOAP11");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-REQUEST-DATA-FORMAT", "SOAP");
		testStub
				.addHttpHeader(msgCtx, "X-EBAY-SOA-RESPONSE-DATA-FORMAT", "XML");

		AbortJobRequest req = new AbortJobRequest();
		System.out.println("Call abortJob...");
		req.setJobId(jobID);
		// invoke
		AbortJobResponse rsp = testStub.abortJob(req, msgCtx);
		AckValue ackValue = rsp.getAck();
		System.out.println("Ack=" + ackValue.value());

		if ("Success".equals(ackValue.value())) {
			System.out.println("Job with jobId=" + req.getJobId()
					+ " has been aborted successfully");
		} else {
			ErrorMessage msg = rsp.getErrorMessage();
			System.out.println("\n" + printErrors(msg));
		}
		return rsp;
	}

	/*
	 * util method, construct the Calendar object from the input string, 
	 * it accepts format of "yyyy-MM-dd"
	 */
	private static XMLGregorianCalendar parseDate(String cal) throws DatatypeConfigurationException {
		GregorianCalendar ret = null;
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		try {
			df.parse(cal);
			String[] parts = cal.split("-");
			ret = (GregorianCalendar) Calendar.getInstance();
			ret.set(Integer.parseInt(parts[0]), Integer.parseInt(parts[1]) - 1,
					Integer.parseInt(parts[2]), 0, 0, 0);
			// get here and we know the format is correct
		} catch (ParseException e) {
			System.out
					.println("ParseException caught when parsing your date string, please fix it and retry.");
			return null;
		}
		DatatypeFactory factory = DatatypeFactory.newInstance();
		XMLGregorianCalendar xmlGregorianCalendar = factory.newXMLGregorianCalendar(ret);
		return xmlGregorianCalendar;
	}

	private static XMLGregorianCalendar parseDateTime(String cal) throws DatatypeConfigurationException {
		GregorianCalendar ret = null;
		SimpleDateFormat df = new SimpleDateFormat("yyyy-mm-dd_hh:mm:ss");
		try {
			df.parse(cal);
			String[] parts = cal.split("_");
			String[] dateItems = parts[0].split("-");
			String[] timeItems = parts[1].split(":");
			ret = (GregorianCalendar) Calendar.getInstance();
			ret.set(Integer.parseInt(dateItems[0]), Integer.parseInt(dateItems[1]) - 1,
					Integer.parseInt(dateItems[2]), Integer.parseInt(timeItems[0]), Integer.parseInt(timeItems[1]), Integer.parseInt(timeItems[2]));
			// get here and we know the format is correct
		} catch(java.lang.NumberFormatException e) {
			System.out
			.println("NumberFormatException caught when parse the DateTime string: "+cal);
		} catch (ParseException e) {
			System.out
					.println("ParseException caught when parsing your date string, please fix it and retry.");
			return null;
		}
		DatatypeFactory factory = DatatypeFactory.newInstance();
		XMLGregorianCalendar xmlGregorianCalendar = factory.newXMLGregorianCalendar(ret);
		return xmlGregorianCalendar;
	}

	/*
	 * main method that is just for quick testing purpose.
	 */
	public static void main(String[] args) throws Exception {
//		 getJobStatus("50000000319", false);
//		 createUploadJob("SetShipmentTrackingInfo");

		getJobs("creationTimeFrom=2008-09-01&creationTimeTo=2008-10-16&jobType=ReviseFixedPriceItem&jobStatus=Aborted");
//		 getJobs("creationTimeFrom=2008-09-01&creationTimeTo=2008-10-14");
//		 abortJob("5000005668");
//		startDownloadJob("ActiveInventoryReport","2008-10-10 12:0:0"); //yyyy-mm-dd hh:mm:ss as profercdi
//		startDownloadJob("ActiveInventoryReport", null);
	}

}
